/*
    File: modbus_slave.h
    Last modify: 26.1.2006
    Programmer: Raul Camaras(raul.camaras@xxxxxxxxxxxxxxxx).
    http://www.itee.uq.edu.au/~listarch/microblaze-uclinux/archive/2006/01/msg00113.html
    Comments:
    Simple modbus TCP slave module. Functions 3 (read) and 16 (write) are implemented:
    Read and write holding registers.
    The port used is the standard for modbus, the 502.

    modpoll -m tcp -p 502 -r 1 -c 10 192.168.10.239
*/

#ifndef _MODBUS_H_
#define _MODBUS_H_

#include "config.h"
#include <avr/io.h>
#include <avr/pgmspace.h>
#include "stack.h"
#include "usart.h"


//#define MODBUS_DEBUG	usart_write		//mit Debugausgabe
#define MODBUS_DEBUG(...)

/*
Public definitions.
*/

#define MODBUS_TCP_IP_PORT    (502)
#define MODBUS_OK               (1)
#define MODBUS_ERROR            (0)


#define MODBUS_REGISTER_CFG_START           0x0

#define MODBUS_REG_OUT_SW_VERSION           0x0
#define MODBUS_REG_OUT_TIMESTAMP            0x1
#define MODBUS_REG_OUT_MCP23017_INPUTS      0x4
#define MODBUS_REG_OUT_MCP23017_OUTPUTS     0x5


#define MODBUS_REGISTER_MCP32017_START      0x10
#define MODBUS_REGISTERS                    MODBUS_REGISTER_MCP32017_START+10



/*
Functions 
*/

void Modbus_InitProtocol(void);
void Modbus_ProcessRequest(uint8_t index);
void Modbus_SetRegister(uint8_t index, uint16_t value);
uint16_t Modbus_GetRegister(uint8_t index);


/*
END Public definitions.
*/


#define MODBUS_BYTE_CNT_INDEX     (5)
#define MODBUS_ID_INDEX           (6)
#define MODBUS_CODE_INDEX         (7)
#define MODBUS_ADDR_HI_INDEX      (8)
#define MODBUS_ADDR_LO_INDEX      (9)
#define MODBUS_QUAN_HI_INDEX     (10)
#define MODBUS_QUAN_LO_INDEX     (11)
#define MODBUS_BYTE_CNT2_INDEX    (8)
#define MODBUS_DATA_BASE_INDEX3   (9)
#define MODBUS_DATA_BASE_INDEX16 (13)
#define MODBUS_BYTE_CNT_INDEX16  (12)

#define MODBUS_EXCEP_CODE_INDEX   (8)
#define MODBUS_CODE_EXCEPTION     (1)
#define MODBUS_QUAN_EXCEPTION     (3)
#define MODBUS_ADDR_EXCEPTION     (2)

#define CODE_READ_MULT_REGISTERS   0x03
#define CODE_WRITE_MULT_REGISTERS  0x10

#endif

